package epsilonStandalone;

import com.google.common.base.Objects;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.LinkedList;
import java.util.List;
import org.eclipse.epsilon.common.parse.problem.ParseProblem;
import org.eclipse.epsilon.common.util.StringProperties;
import org.eclipse.epsilon.egl.EglFileGeneratingTemplateFactory;
import org.eclipse.epsilon.egl.EglTemplateFactoryModuleAdapter;
import org.eclipse.epsilon.emc.emf.EmfModel;
import org.eclipse.epsilon.eol.EolModule;
import org.eclipse.epsilon.eol.IEolExecutableModule;
import org.eclipse.epsilon.eol.IEolModule;
import org.eclipse.epsilon.eol.exceptions.EolRuntimeException;
import org.eclipse.epsilon.eol.exceptions.models.EolModelLoadingException;
import org.eclipse.epsilon.eol.execute.context.FrameStack;
import org.eclipse.epsilon.eol.execute.context.IEolContext;
import org.eclipse.epsilon.eol.execute.context.Variable;
import org.eclipse.epsilon.eol.models.IModel;
import org.eclipse.epsilon.eol.models.IRelativePathResolver;
import org.eclipse.epsilon.eol.models.ModelRepository;
import org.eclipse.epsilon.etl.EtlModule;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;
import org.eclipse.xtext.xbase.lib.Exceptions;
import org.eclipse.xtext.xbase.lib.InputOutput;

@SuppressWarnings("all")
public class EpsilonBase {
  /**
   * Reads model from file given by URI and applies EOL script and returns the modified model without storing it to the disk
   */
  public static EmfModel applyEolScript(final String modelName, final String modelURI, final String metaModelURI, final String eolScriptPath, final List<Variable> eolParameters) {
    try {
      EmfModel model = EpsilonBase.createEmfModel(modelName, modelURI, metaModelURI, true, false);
      return EpsilonBase.applyEolScript(model, eolScriptPath, eolParameters);
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  /**
   * Applies EOL script on Emf model and returns the modified model without storing it to the disk
   */
  public static EmfModel applyEolScript(final EmfModel model, final String eolScriptPath, final List<Variable> eolParameters) {
    try {
      EolModule _eolModule = new EolModule();
      LinkedList<IModel> _newLinkedList = CollectionLiterals.<IModel>newLinkedList(model);
      EpsilonBase.prepareAndExecute(_eolModule, eolScriptPath, _newLinkedList, eolParameters);
      return model;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public static Object applyEtlScript(final EmfModel input, final EmfModel output, final String etlScriptPath, final List<Variable> eolParameters) {
    try {
      EtlModule _etlModule = new EtlModule();
      LinkedList<IModel> _newLinkedList = CollectionLiterals.<IModel>newLinkedList(input, output);
      EpsilonBase.prepareAndExecute(_etlModule, etlScriptPath, _newLinkedList, eolParameters);
      return output;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public static String applyEglScript(final EmfModel model, final String eglScriptPath, final List<Variable> eglParameters) {
    try {
      EglFileGeneratingTemplateFactory _eglFileGeneratingTemplateFactory = new EglFileGeneratingTemplateFactory();
      EglTemplateFactoryModuleAdapter eglModule = new EglTemplateFactoryModuleAdapter(_eglFileGeneratingTemplateFactory);
      LinkedList<IModel> _newLinkedList = CollectionLiterals.<IModel>newLinkedList(model);
      Object file_contents = EpsilonBase.prepareAndExecute(eglModule, eglScriptPath, _newLinkedList, eglParameters);
      return ((String) file_contents);
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  /**
   * Executes Epsilon code over models
   */
  private static Object prepareAndExecute(final IEolExecutableModule module, final String source, final List<IModel> models, final List<Variable> parameters) throws Exception {
    URI resURI = EpsilonBase.getFileURI(source);
    module.parse(resURI);
    List<ParseProblem> _parseProblems = module.getParseProblems();
    int _size = _parseProblems.size();
    boolean _greaterThan = (_size > 0);
    if (_greaterThan) {
      System.err.println("Parse errors occurred...");
      List<ParseProblem> _parseProblems_1 = module.getParseProblems();
      for (final ParseProblem problem : _parseProblems_1) {
        String _string = problem.toString();
        System.err.println(_string);
      }
      return null;
    }
    for (final IModel model : models) {
      IEolContext _context = module.getContext();
      ModelRepository _modelRepository = _context.getModelRepository();
      _modelRepository.addModel(model);
    }
    for (final Variable parameter : parameters) {
      IEolContext _context_1 = module.getContext();
      FrameStack _frameStack = _context_1.getFrameStack();
      _frameStack.put(parameter);
    }
    Object result = EpsilonBase.execute(module);
    for (final IModel model_1 : models) {
      IEolContext _context_2 = module.getContext();
      ModelRepository _modelRepository_1 = _context_2.getModelRepository();
      _modelRepository_1.removeModel(model_1);
    }
    return result;
  }
  
  private static Object execute(final IEolExecutableModule module) throws EolRuntimeException {
    return module.execute();
  }
  
  /**
   * Reads emf model object from file
   */
  public static EmfModel readEmfModel(final String name, final String modelURI, final String metamodelURI) throws EolModelLoadingException, URISyntaxException {
    EmfModel emfModel = new EmfModel();
    StringProperties properties = new StringProperties();
    properties.put(EmfModel.PROPERTY_NAME, name);
    URI _fileURI = EpsilonBase.getFileURI(metamodelURI);
    properties.put(EmfModel.PROPERTY_FILE_BASED_METAMODEL_URI, _fileURI);
    URI _fileURI_1 = EpsilonBase.getFileURI(modelURI);
    properties.put(EmfModel.PROPERTY_MODEL_URI, _fileURI_1);
    String _plus = (Boolean.valueOf(true) + "");
    properties.put(EmfModel.PROPERTY_READONLOAD, _plus);
    String _plus_1 = (Boolean.valueOf(false) + "");
    properties.put(EmfModel.PROPERTY_STOREONDISPOSAL, _plus_1);
    emfModel.load(properties, ((IRelativePathResolver) null));
    return emfModel;
  }
  
  /**
   * Creates emf model object if a file addressed over uri does not exist
   */
  public static EmfModel createEmfModel(final String name, final String modelURI, final String metamodelURI, final boolean readOnLoad, final boolean storeOnDisposal) throws EolModelLoadingException, URISyntaxException {
    EmfModel emfModel = new EmfModel();
    StringProperties properties = new StringProperties();
    properties.put(EmfModel.PROPERTY_NAME, name);
    URI _fileURI = EpsilonBase.getFileURI(metamodelURI);
    properties.put(EmfModel.PROPERTY_FILE_BASED_METAMODEL_URI, _fileURI);
    Object _xtrycatchfinallyexpression = null;
    try {
      _xtrycatchfinallyexpression = EpsilonBase.getFileURI(modelURI);
    } catch (final Throwable _t) {
      if (_t instanceof Exception) {
        final Exception e = (Exception)_t;
        _xtrycatchfinallyexpression = modelURI;
      } else {
        throw Exceptions.sneakyThrow(_t);
      }
    }
    properties.put(EmfModel.PROPERTY_MODEL_URI, _xtrycatchfinallyexpression);
    String _plus = (Boolean.valueOf(readOnLoad) + "");
    properties.put(EmfModel.PROPERTY_READONLOAD, _plus);
    String _plus_1 = (Boolean.valueOf(storeOnDisposal) + "");
    properties.put(EmfModel.PROPERTY_STOREONDISPOSAL, _plus_1);
    emfModel.load(properties, ((IRelativePathResolver) null));
    return emfModel;
  }
  
  public static EmfModel createEmfModelByURI(final String name, final String modelURI, final String metamodelURI, final boolean readOnLoad, final boolean storeOnDisposal) throws EolModelLoadingException, URISyntaxException {
    EmfModel emfModel = new EmfModel();
    StringProperties properties = new StringProperties();
    properties.put(EmfModel.PROPERTY_NAME, name);
    URI _fileURI = EpsilonBase.getFileURI(metamodelURI);
    properties.put(EmfModel.PROPERTY_FILE_BASED_METAMODEL_URI, _fileURI);
    properties.put(EmfModel.PROPERTY_MODEL_URI, modelURI);
    String _plus = (Boolean.valueOf(readOnLoad) + "");
    properties.put(EmfModel.PROPERTY_READONLOAD, _plus);
    String _plus_1 = (Boolean.valueOf(storeOnDisposal) + "");
    properties.put(EmfModel.PROPERTY_STOREONDISPOSAL, _plus_1);
    emfModel.load(properties, ((IRelativePathResolver) null));
    return emfModel;
  }
  
  public static EmfModel createEmfModelByMetaModelURI(final String name, final String model, final String metamodel, final boolean readOnLoad, final boolean storeOnDisposal) throws EolModelLoadingException, URISyntaxException {
    EmfModel emfModel = new EmfModel();
    StringProperties properties = new StringProperties();
    properties.put(EmfModel.PROPERTY_NAME, name);
    properties.put(EmfModel.PROPERTY_METAMODEL_URI, metamodel);
    URI _fileURI = EpsilonBase.getFileURI(model);
    String _string = _fileURI.toString();
    properties.put(EmfModel.PROPERTY_MODEL_URI, _string);
    String _plus = (Boolean.valueOf(readOnLoad) + "");
    properties.put(EmfModel.PROPERTY_READONLOAD, _plus);
    String _plus_1 = (Boolean.valueOf(storeOnDisposal) + "");
    properties.put(EmfModel.PROPERTY_STOREONDISPOSAL, _plus_1);
    emfModel.load(properties, ((IRelativePathResolver) null));
    return emfModel;
  }
  
  private static URI getFileURI(final String fileName) throws URISyntaxException {
    InputOutput.<String>println(("searching URI for file: " + fileName));
    URI binUri = null;
    URL res = null;
    try {
      URL _resource = EpsilonBase.class.getResource(fileName);
      res = _resource;
    } catch (final Throwable _t) {
      if (_t instanceof Exception) {
        final Exception e = (Exception)_t;
      } else {
        throw Exceptions.sneakyThrow(_t);
      }
    }
    boolean _equals = Objects.equal(res, null);
    if (_equals) {
      try {
        URL _uRL = new URL(fileName);
        res = _uRL;
      } catch (final Throwable _t_1) {
        if (_t_1 instanceof Exception) {
          final Exception e_1 = (Exception)_t_1;
        } else {
          throw Exceptions.sneakyThrow(_t_1);
        }
      }
    }
    boolean _equals_1 = Objects.equal(res, null);
    if (_equals_1) {
      try {
        URL _uRL_1 = new URL(("file:" + fileName));
        res = _uRL_1;
      } catch (final Throwable _t_2) {
        if (_t_2 instanceof Exception) {
          final Exception e_2 = (Exception)_t_2;
        } else {
          throw Exceptions.sneakyThrow(_t_2);
        }
      }
    }
    InputOutput.<String>println(("found resource: " + res));
    URI _uRI = res.toURI();
    binUri = _uRI;
    InputOutput.<String>println(("binUri: " + binUri));
    URI uri = null;
    String _string = binUri.toString();
    int _indexOf = _string.indexOf("bin");
    boolean _greaterThan = (_indexOf > (-1));
    if (_greaterThan) {
      String _string_1 = binUri.toString();
      String _replaceAll = _string_1.replaceAll("bin", "src");
      URI _uRI_1 = new URI(_replaceAll);
      uri = _uRI_1;
    } else {
      uri = binUri;
    }
    InputOutput.<String>println(("\t... resulted in: " + uri));
    return uri;
  }
}
